@@ -1,4 +1,7 @@
+from datetime import datetime
+
 import pandas as pd
+
 from .model import Article, Comment
 
 
@@ -96,6 +99,30 @@ class PTTData:
             return None
         return self.__comment_list[-1]
 
+    # return all article date
+    def get_date_from_article(self) -> list[datetime]:
+        date_list = list()
+        for article in self.__article_list:
+            date = article.post_time.replace(hour=0, minute=0, second=0, microsecond=0)
+            if date not in date_list:
+                date_list.append(date)
+        return date_list
+
+    def delete_data_by_date(self, start_time: datetime, end_time: datetime) -> None:
+        pos = 0
+        while pos < len(self.__article_list):
+            time = self.__article_list[pos].post_time
+            if time < start_time or time > end_time:
+                del self.__article_list[pos]
+            else:
+                pos += 1
+        self.__article_list.sort(key=lambda x: x.post_time)
+
+        del self.__comment_list
+        self.__comment_list = list()
+        for article in self.__article_list:
+            self.__comment_list.extend(article.comment_list)
+
 
 def main():
     ptt = PTTData()
